package kr.co.vividnext.sodalive.admin.point

import com.querydsl.core.types.dsl.DateTimePath
import com.querydsl.core.types.dsl.Expressions
import com.querydsl.core.types.dsl.StringTemplate
import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.point.PointRewardPolicy
import kr.co.vividnext.sodalive.point.QPointRewardPolicy.pointRewardPolicy
import org.springframework.data.jpa.repository.JpaRepository
import java.time.LocalDateTime

interface PointPolicyRepository : JpaRepository<PointRewardPolicy, Long>, PointPolicyQueryRepository

interface PointPolicyQueryRepository {
    fun getAll(offset: Long, limit: Long): List<GetPointRewardPolicyResponse>
}

class PointPolicyQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory
) : PointPolicyQueryRepository {
    override fun getAll(offset: Long, limit: Long): List<GetPointRewardPolicyResponse> {
        return queryFactory
            .select(
                QGetPointRewardPolicyResponse(
                    pointRewardPolicy.id,
                    pointRewardPolicy.title,
                    pointRewardPolicy.actionType,
                    pointRewardPolicy.threshold,
                    pointRewardPolicy.pointAmount,
                    getFormattedDate(pointRewardPolicy.startDate),
                    getFormattedDate(pointRewardPolicy.endDate),
                    pointRewardPolicy.isActive
                )
            )
            .from(pointRewardPolicy)
            .orderBy(pointRewardPolicy.isActive.desc(), pointRewardPolicy.startDate.desc())
            .offset(offset)
            .limit(limit)
            .fetch()
    }

    private fun getFormattedDate(dateTimePath: DateTimePath<LocalDateTime>): StringTemplate {
        return Expressions.stringTemplate(
            "DATE_FORMAT({0}, {1})",
            Expressions.dateTimeTemplate(
                LocalDateTime::class.java,
                "CONVERT_TZ({0},{1},{2})",
                dateTimePath,
                "UTC",
                "Asia/Seoul"
            ),
            "%Y-%m-%d %H:%i"
        )
    }
}
